"""Testing examples."""

from limax.example import example_limax_dir, example_limax_file


def test_example_limax_dir() -> None:
    """Run example 1."""
    example_limax_dir()


def test_example_limax_file() -> None:
    """Run example 2."""
    example_limax_file()
